#include "curve.h"
using namespace Imagine;

// Nouvelle courbe
Curve newCurve(int nb) {
    Curve c;
    c.n = nb;
    c.x = new float[nb];
    c.y = new float[nb];
    return c;
}

// Liberation de la memoire
void deleteCurve(Curve c) {
    delete [] c.x;
    delete [] c.y;
}

// Echantillonnage regulier en x
void regularx(Curve c, float min, float max) {
    float pas = (max-min)/c.n;
    for(int i=0; i<c.n; i++)
        c.x[i] = min + i*pas;
}

// Fonction affine pour affichage
doubleAffine fill(float xmin, float xmax, float ymin, float ymax) {
    doubleAffine aff;
    aff.ax = (taille-2*marge)/float(xmax-xmin);
    aff.bx = -aff.ax*xmin + marge;
    aff.ay = -(taille-2*marge)/float(ymax-ymin);
    aff.by = -aff.ay*ymin + taille - marge;
    return aff;
}

// Calcule min et max d'un tableau
void minmax(float tab[], int n, float& min, float& max) {
    min = max = tab[0];
    for(int i=0; i<n; i++) {
        if(min > tab[i])
            min = tab[i];
        if(max < tab[i])
            max = tab[i];
    }
}

// Calcule une fonction affine d'affichage appropriee pour une courbe
doubleAffine fit(Curve c) {
    float xmin, xmax, ymin, ymax;
    minmax(c.x, c.n, xmin, xmax);
    minmax(c.y, c.n, ymin, ymax);
    doubleAffine aff = fill(xmin, xmax, ymin, ymax);
    return aff;
}

// Trace d'une courbe
void drawCurve(Curve c, doubleAffine aff, Color col) {
    float xp = aff.ax*c.x[0]+aff.bx, yp = aff.ay*c.y[0]+aff.by;
    for(int i=1; i<c.n; i++) {
        float x = aff.ax*c.x[i]+aff.bx, y = aff.ay*c.y[i]+aff.by;
        drawLine(xp, yp, x, y, col);
        xp = x;
        yp = y;
    }
}

// Index du point de la courbe le plus proche du point (x,y) de l'affichage
int indexClick(Curve c, doubleAffine aff, int x, int y) {
    int imin = 0, min=2*taille*taille;
    for(int i=0; i<c.n; i++) {
        float dx = x - (aff.ax*c.x[i]+aff.bx);
        float dy = y - (aff.ay*c.y[i]+aff.by);
        float d = dx*dx+dy*dy;
        if(min > d) {
            min = d;
            imin = i;
        }
    }       
    return imin;
}
