#include "curve.h"
#include <iostream>
#include <cmath>
using namespace Imagine;
using namespace std;

// Trace de la fonction y = 10 sqrt(x)
void racine() {
    float xp=0, yp=0;
    for(int i=0; i<=100; i++) {
        float x=i;
        float y=sqrt(x)*10;
        drawLine(marge+4*xp, taille-marge-yp,
                 marge+4*x , taille-marge-y, RED);
        xp=x;
        yp=y;
    }
}

// Trace du cadre et des tics
void traceAxes() {
    drawRect(marge, marge, taille-2*marge, taille-2*marge, BLACK); // cadre
    int pas = (taille-2*marge)/10;
    for(int x=pas; x<taille-2*marge; x+=pas) { // tics verticaux
        drawLine(marge+x, marge-2, marge+x, marge+2, BLUE);
        drawLine(marge+x, taille-marge-2, marge+x, taille-marge+2, BLUE);
    }
    for(int y=pas; y<taille-2*marge; y+=pas) { // tics horizontaux
        drawLine(marge-2, marge+y, marge+2, marge+y, BLUE);
        drawLine(taille-marge-2, marge+y, taille-marge+2, marge+y, BLUE);
    }
}

// Clic des points de la courbe
void clicPoints(Curve c, doubleAffine aff) {
    int x, y;
    while(getMouse(x,y) != 3) {
        int i = indexClick(c, aff, x, y);
        clearWindow();
        drawCurve(c, aff, GREEN);
        fillCircle(aff.ax*c.x[i]+aff.bx,
                   aff.ay*c.y[i]+aff.by, 3, RED);
        traceAxes();
        cout << c.x[i] << ' ' << c.y[i] << endl;
    }
}

// Profil RGB d'image le long d'un segment
void profilImage(Window wCurve) {
    // Chargement d'image
    Color* col;
    int w, h;
    if(! loadColorImage(srcPath("image.png"), col, w, h)) {
        cerr << "Impossible to load image " << srcPath("image.png") << endl;
        return;
    }

    // Trace la ligne cliquee par l'utilisateur
    setActiveWindow(openWindow(w,h));
    putColorImage(0,0, col, w, h);
    int x1, y1, x2, y2;
    getMouse(x1, y1);
    getMouse(x2, y2);
    drawLine(x1, y1, x2, y2, WHITE);

    // Echantillonnage
    float dx=1, dy=1;
    int n=0;
    if(abs(x2-x1)>=abs(y2-y1)) {
        dy = (y2-y1)/(x2-x1);
        n = abs(x2-x1)+1;
    } else {
        dx = (x2-x1)/(y2-y1);
        n = abs(y2-y1)+1;
    }
    Curve r=newCurve(n), g=newCurve(n), b=newCurve(n);
    for(int i=0; i<n; i++) {
        r.x[i] = g.x[i] = b.x[i] = i;
        Color c = col[w*(int)(y1+i*dy) + (int)(x1+i*dx)];
        r.y[i] = c.r(); g.y[i] = c.g(); b.y[i] = c.b();
    }
    delete [] col; // On n'a plus besoin de l'image

    // Reglage affichage
    doubleAffine aff = fill(0,n,0,255);

    // Trace les courbes de profil
    setActiveWindow(wCurve);
    clearWindow();
    traceAxes();
    drawCurve(r, aff, RED);
    drawCurve(g, aff, GREEN);
    drawCurve(b, aff, BLUE);

    // On n'oublie pas de liberer la memoire
    deleteCurve(r);
    deleteCurve(g);
    deleteCurve(b);
}

int main() {
    Window wCurve = openWindow(taille, taille, "Curves");
    racine();
    click();
    traceAxes();
    click();
    clearWindow();

    cout << "Entrez le nombre de points: ";
    int n; cin >> n;
    Curve c = newCurve(n);
    regularx(c, -5, 5);
    for(int i=0; i<c.n; i++)
        c.y[i] = sin(c.x[i]*c.x[i]);
    doubleAffine aff = fit(c);
    drawCurve(c, aff, GREEN);
    traceAxes();

    clicPoints(c, aff);
    deleteCurve(c);

    profilImage(wCurve);

    endGraphics();
    return 0;
}
